@@ -1010,8 +1010,13 @@ class Sequence(Generic[T], Iterable[T]):
         >>> it(lst).distinct()
         [1, 2, 3]
 
+         Example 2:
+        >>> lst = [(1, 'A'), (1, 'A'), (1, 'A'), (2, 'A'), (3, 'C'), (3, 'D')]
+        >>> it(lst).distinct().sorted()
+        [(1, 'A'), (2, 'A'), (3, 'C'), (3, 'D')]
+
         """
-        return Sequence(set(self))
+        return DistinctSequence(self)
     
     def distinct_by(self, key_selector: Callable[[T], K]) -> Sequence[T]:
         """
@@ -1020,10 +1025,10 @@ class Sequence(Generic[T], Iterable[T]):
          Example 1:
         >>> lst = [1, 2, 3, 1, 2, 3]
         >>> it(lst).distinct_by(lambda x: x%2)
-        [3, 2]
+        [1, 2]
 
         """
-        return Sequence(self.associate_by(key_selector).values())
+        return DistinctSequence(self, key_selector)
     
     def reduce(self, accumulator: Callable[[R, T], R], initial: Optional[R] = None) -> T:
         """
@@ -1972,7 +1977,21 @@ class MergingWithNextSequence(Sequence[V]):
             pass
 
 
-class GroupingSequence(Sequence[Tuple[K, List[T]]]):
+class DistinctSequence(Sequence[T]):
+    def __init__(self, iterable: Iterable[T], key_selector: Optional[Callable[[T], K]]=None) -> None:
+        super().__init__(iterable)
+        self._key_selector = key_selector
+    
+    def __do_iter__(self) -> Iterator[T]:
+        seen = set()
+        for e in self._iter:
+            k = self._key_selector(e) if self._key_selector else e
+            if k not in seen:
+                seen.add(k)
+                yield e
+
+
+class GroupingSequence(Sequence[Tuple[K, Sequence[T]]]):
     def __init__(self, iterable: Iterable[T], key_func: Callable[[T], K]) -> None:
         super().__init__(iterable)
         self._key_func = key_func
@@ -1982,19 +2001,19 @@ class GroupingSequence(Sequence[Tuple[K, List[T]]]):
         return self.map(lambda x: x[0])
     
     @property
-    def values(self) -> Sequence[List[T]]:
+    def values(self) -> Sequence[Sequence[T]]:
         return self.map(lambda x: x[1])
     
     @property
-    def items(self) -> Sequence[Tuple[K, List[T]]]:
+    def items(self) -> Sequence[Tuple[K, Sequence[T]]]:
         return self
     
-    def __do_iter__(self) -> Iterator[Tuple[K, List[T]]]:
+    def __do_iter__(self) -> Iterator[Tuple[K, Sequence[T]]]:
         from collections import defaultdict
         d = defaultdict(list)
         for e in self._iter:
             d[self._key_func(e)].append(e)
-        yield from d.items()
+        yield from it(d.items()).map(lambda x: (x[0], it(x[1])))
 
 
 class CombinationSequence(Sequence[Tuple[T, ...]]):
